//
// Stolen from https://github.com/adamschwartz/log
// Reorganized for use with browserify and for maintaining as a javascript
// library
//
function log() {
  var args = [];

  [].slice.call(arguments).forEach(function(arg) {
    if (typeof arg === 'string') {
      return args = args.concat(stringToArgs(arg));
    }

    args.push(arg);
  });
  console.log.apply(console, args);
};

var formats = [
  {
    regex: /\*([^\*)]+)\*/,
    replacer: function(m, p1) {
      return "%c" + p1 + "%c";
    },
    styles: function() {
      return ['font-style: italic', ''];
    }
  }, {
    regex: /\_([^\_)]+)\_/,
    replacer: function(m, p1) {
      return "%c" + p1 + "%c";
    },
    styles: function() {
      return ['font-weight: bold', ''];
    }
  }, {
    regex: /\`([^\`)]+)\`/,
    replacer: function(m, p1) {
      return "%c" + p1 + "%c";
    },
    styles: function() {
      return ['background: rgb(255, 255, 219); padding: 1px 5px; border: 1px solid rgba(0, 0, 0, 0.1)', ''];
    }
  }, {
    regex: /\[c\=\"([^\")]+)\"\]([^\[)]+)\[c\]/,
    replacer: function(m, p1, p2) {
      return "%c" + p2 + "%c";
    },
    styles: function(match) {
      return [match[1], ''];
    }
  }
];

function hasMatches(str) {
  var _hasMatches = false;
  formats.forEach(function(format) {
    if (format.regex.test(str)) {
      _hasMatches = true;
    }
  });
  return _hasMatches;
};

function getOrderedMatches(str) {
  var matches = [];
  formats.forEach(function(format) {
    var match = str.match(format.regex);
    if (match) {
      matches.push({
        format: format,
        match: match
      });
    }
  });
  return matches.sort(function(a, b) {
    return a.match.index - b.match.index;
  });
};

function stringToArgs(str) {
  var styles = [], firstMatch, matches, styles;

  while (hasMatches(str)) {
    matches = getOrderedMatches(str);
    firstMatch = matches[0];
    str = str.replace(firstMatch.format.regex, firstMatch.format.replacer);
    styles = styles.concat(firstMatch.format.styles(firstMatch.match));
  }
  return [str].concat(styles);
};


function isSafari() {
  return /Safari/.test(navigator.userAgent) && /Apple Computer/.test(navigator.vendor);
};

function isIE() {
  return /MSIE/.test(navigator.userAgent);
};

function safariSupport() {
  var m = navigator.userAgent.match(/AppleWebKit\/(\d+)\.(\d+)(\.|\+|\s)/);
  if (!m) {
    return false;
  }
  return 537.38 >= parseInt(m[1], 10) + (parseInt(m[2], 10) / 100);
};

if ((isSafari() && !safariSupport()) || isIE()) {
  module.exports = console.log.bind(console);
}
else {
  module.exports = log;
}

exports.log = log;
