var test = require('tape');

var loggins = require('./loggins');

var logger;

test('create a logger', function (t) {
  t.doesNotThrow(function () {
    logger = loggins();
  }, 'creating a logger does not throw');

  t.ok(logger._pretty, 'logger._pretty exists');
  t.ok(logger.raw, 'logger.raw exists');
  t.ok(logger.log, 'logger.log exists');
  t.ok(logger.logging, 'logger.logging exists');
  [
    'debug',
    'info',
    'warn',
    'error'
  ].forEach(function (lvl) {
    t.ok(logger[lvl], 'logger.' + lvl + ' exists');
  });
  t.end();
});

test('reading/writing with a logger works (pretty)', function (t) {
  logger.once('data', function (buff) {
    t.equal(
      buff.toString(),
      loggins.colorize('debug', 'grey') + ': a\n',
      'debug: a'
    );
    logger.once('data', function (buff) {
      t.equal(
        buff.toString(),
        loggins.colorize('info', 'green') + ': b\n',
        'info: b'
      );
      logger.once('data', function (buff) {
        t.equal(
          buff.toString(),
          loggins.colorize('warn', 'yellow') + ': c\n',
          'warn: c'
        );
        logger.once('data', function (buff) {
          t.equal(
            buff.toString(),
            loggins.colorize('error', 'red') + ': d\n',
            'error: d'
          );
          t.end();
        });

        logger.error('d');
      });
      logger.warn('c');
    });
    logger.info('b');
  });
  logger.debug('a');
});

test('reading/writing with a logger works (raw)', function (t) {
  var body;

  logger.raw.once('data', function (buff) {
    t.doesNotThrow(function () {
      body = JSON.parse(buff.toString());
    }, 'debug log is valid JSON');
    t.equal(body.level, 'debug', 'level: debug');
    t.equal(body.message, 'a\n', 'message: a');
    t.notEqual(
      String(new Date(body.timestamp)),
      'Invalid Date',
      'timestamp: a date'
    );

    logger.raw.once('data', function (buff) {
      t.doesNotThrow(function () {
        body = JSON.parse(buff.toString());
      }, 'info log is valid JSON');
      t.equal(body.level, 'info', 'level: info');
      t.equal(body.message, 'b\n', 'message: b');
      t.notEqual(
        String(new Date(body.timestamp)),
        'Invalid Date',
        'timestamp: a date'
      );

      logger.raw.once('data', function (buff) {
        t.doesNotThrow(function () {
          body = JSON.parse(buff.toString());
        }, 'warn log is valid JSON');
        t.equal(body.level, 'warn', 'level: warn');
        t.equal(body.message, 'c\n', 'message: c');
        t.notEqual(
          String(new Date(body.timestamp)),
          'Invalid Date',
          'timestamp: a date'
        );

        logger.raw.once('data', function (buff) {
          t.doesNotThrow(function () {
            body = JSON.parse(buff.toString());
          }, 'error log is valid JSON');
          t.equal(body.level, 'error', 'level: error');
          t.equal(body.message, 'd\n', 'message: d');
          t.notEqual(
            String(new Date(body.timestamp)),
            'Invalid Date',
            'timestamp: a date'
          );

          t.end();
        });

        logger.error('d');
      });
      logger.warn('c');
    });
    logger.info('b');
  });
  logger.debug('a');
});

test('reading a logger works (scoped)', function (t) {
  logger.debug.once('data', function (buff) {
    t.equal(buff.toString(), 'a\n', 'debug: a');
    logger.info.once('data', function (buff) {
      t.equal(buff.toString(), 'b\n', 'info: b');
      logger.warn.once('data', function (buff) {
        t.equal(buff.toString(), 'c\n', 'warn: c');
        logger.error.once('data', function (buff) {
          t.equal(buff.toString(), 'd\n', 'error: d');
          t.end();
        });

        logger.error('d');
      });
      logger.warn('c');
    });
    logger.info('b');
  });
  logger.debug('a');
});

test('writing to logref works', function (t) {
  var logref = logger.logging('logref');

  logger.once('data', function (buff) {
    t.equal(
      buff.toString(),
     'logref: Good morning starshine ! The Earth says hello !\n',
     'logref: Good morning starshine ! The Earth says hello !'
    );
    t.end();
  });

  logref(
    'Good morning %target ! The Earth says %greeting !',
    { target: 'starshine', greeting: 'hello' }
  );
});
