var test = require('tap').test,
    pinocchio = require('./');

test('Strings', function (t) {

  var err = pinocchio.isARealError('That\'s not an error!');

  t.ok(err instanceof Error, 'Is an instance of Error');
  t.equal(err.message, 'That\'s not an error!', 'Has the expected error message');
  t.ok(hasFail(err, 'is a string'), 'Reports that input was a string');
  t.end();
});

test('JSON string', function (t) {
  var err = pinocchio.isARealError(JSON.stringify({
    code: 200,
    message: 'a JSON string with a message'
  }));

  t.ok(err instanceof Error, 'Is an instance of Error');
  t.equal(err.message, 'a JSON string with a message', 'Has the expected error message');
  t.equal(err.code, 200, 'Has expected metadata properties');
  t.ok(hasFail(err, 'is a string' ), 'Reports that input was a string');
  t.ok(hasFail(err, 'is a JSON-encoded string'), 'Reports that input was a JSON blob');
  t.end();
});

test('JSON blob inside a string', function (t) {
  var err = pinocchio.isARealError('I\'M HELPING!! ' + JSON.stringify({ payload: 'ponyyyyyy' }));

  t.ok(err instanceof Error, 'Is an instance of Error');
  t.equal(err.payload, 'ponyyyyyy', 'Has expected metadata properties');
  t.ok(hasFail(err, 'is a string' ), 'Reports that input was a string');
  t.ok(hasFail(err, 'is a string with a JSON fragment inside it' ), 'Reports that input contained JSON fragment');
  t.end();
});

test('A javascript object', function (t) {
  var err = pinocchio.isARealError({ code: 404, message: 'not_found'});

  t.ok(err instanceof Error, 'Is an instance of Error');
  t.equal(err.message, 'not_found', 'Has the expected error message');
  t.equal(err.code, 404, 'Has expected metadata properties');
  t.ok(hasFail(err, 'is a non-error object'), 'Reports that input was a non-error object');
  t.end();
});

function hasFail(err, expected) {
  return err.fails && err.fails.indexOf(expected) !== -1;
};
